"""
No, this is not a real DB, but it is a big blob of global mutable state...
...like a DB.
"""
import collections

from . import datatypes


db = {
    'user': None,
    'accounts': collections.OrderedDict(),
    'transactions': {},
    'merchant_groups': {},
    'merchants': {},
}


def init(num_accounts, currency):
    db['user'] = datatypes.User()

    merchant_groups = [datatypes.MerchantGroup() for _ in range(100)]
    db['merchant_groups'] = dict(
        (m.merchant_group_id, m) for m in merchant_groups
    )

    merchants = [datatypes.Merchant(merchant_groups) for _ in range(200)]
    db['merchants'] = dict((m.merchant_id, m) for m in merchants)

    for _ in range(num_accounts):
        account = datatypes.Account(db['user'], currency)
        db['accounts'][account.account_id] = account

        transactions = sorted([
            datatypes.Transaction(account, merchants) for _ in range(1000)
        ], key=lambda x: x.created)

        db['transactions'][account.account_id] = collections.OrderedDict([
            (t.transaction_id, t) for t in transactions
        ])



import emoji
import random

from faker import Faker


fake = Faker()

CATEGORIES = ( 'general', 'eating_out', 'expenses', 'transport', 'cash',
'bills', 'entertainment', 'shopping', 'holidays', 'groceries', )

EMOJI = ( ':bamboo:', ':gift_heart:', ':dolls:', ':school_satchel:',
':mortar_board:', ':flags:', ':fireworks:', ':sparkler:', ':wind_chime:',
':rice_scene:', ':jack_o_lantern:', ':ghost:', ':santa:', ':christmas_tree:',
':gift:', ':bell:', ':no_bell:', ':tanabata_tree:', ':tada:', ':confetti_ball:',
':balloon:', ':crystal_ball:', ':cd:', ':dvd:', ':floppy_disk:', ':camera:',
':video_camera:', ':movie_camera:', ':computer:', ':tv:', ':iphone:', ':phone:',
':telephone:', ':telephone_receiver:', ':pager:', ':fax:', ':minidisc:',
':vhs:', ':sound:', ':speaker:', ':mute:', ':loudspeaker:', ':mega:',
':hourglass:', ':hourglass_flowing_sand:', ':alarm_clock:', ':watch:',
':radio:', ':satellite:', ':loop:', ':mag:', ':mag_right:', ':unlock:',
':lock:', ':lock_with_ink_pen:', ':closed_lock_with_key:', ':key:', ':bulb:',
':flashlight:', ':high_brightness:', ':low_brightness:', ':electric_plug:',
':battery:', ':calling:', ':email:', ':mailbox:', ':postbox:', ':bath:',
':bathtub:', ':shower:', ':toilet:', ':wrench:', ':nut_and_bolt:', ':hammer:',
':seat:', ':moneybag:', ':yen:', ':dollar:', ':pound:', ':euro:',
':credit_card:', ':money_with_wings:', ':e-mail:', ':inbox_tray:',
':outbox_tray:', ':envelope:', ':incoming_envelope:', ':postal_horn:',
':mailbox_closed:', ':mailbox_with_mail:', ':mailbox_with_no_mail:',
':package:', ':door:', ':smoking:', ':bomb:', ':gun:', ':hocho:', ':pill:',
':syringe:', ':page_facing_up:', ':page_with_curl:', ':bookmark_tabs:',
':bar_chart:', ':chart_with_upwards_trend:', ':chart_with_downwards_trend:',
':scroll:', ':clipboard:', ':calendar:', ':date:', ':card_index:',
':file_folder:', ':open_file_folder:', ':scissors:', ':pushpin:', ':paperclip:',
':black_nib:', ':pencil2:', ':straight_ruler:', ':triangular_ruler:',
':closed_book:', ':green_book:', ':blue_book:', ':orange_book:', ':notebook:',
':notebook_with_decorative_cover:', ':ledger:', ':books:', ':bookmark:',
':name_badge:', ':microscope:', ':telescope:', ':newspaper:', ':football:',
':basketball:', ':soccer:', ':baseball:', ':tennis:', ':8ball:',
':rugby_football:', ':bowling:', ':golf:', ':mountain_bicyclist:',
':bicyclist:', ':horse_racing:', ':snowboarder:', ':swimmer:', ':surfer:',
':ski:', ':spades:', ':hearts:', ':clubs:', ':diamonds:', ':gem:', ':ring:',
':trophy:', ':musical_score:', ':musical_keyboard:', ':violin:',
':space_invader:', ':video_game:', ':black_joker:', ':flower_playing_cards:',
':game_die:', ':dart:', ':mahjong:', ':clapper:', ':memo:', ':pencil:',
':book:', ':art:', ':microphone:', ':headphones:', ':trumpet:', ':saxophone:',
':guitar:', ':shoe:', ':sandal:', ':high_heel:', ':lipstick:', ':boot:',
':shirt:', ':tshirt:', ':necktie:', ':womans_clothes:', ':dress:',
':running_shirt_with_sash:', ':jeans:', ':kimono:', ':bikini:', ':ribbon:',
':tophat:', ':crown:', ':womans_hat:', ':mans_shoe:', ':closed_umbrella:',
':briefcase:', ':handbag:', ':pouch:', ':purse:', ':eyeglasses:',
':fishing_pole_and_fish:', ':coffee:', ':tea:', ':sake:', ':baby_bottle:',
':beer:', ':beers:', ':cocktail:', ':tropical_drink:', ':wine_glass:',
':fork_and_knife:', ':pizza:', ':hamburger:', ':fries:', ':poultry_leg:',
':meat_on_bone:', ':spaghetti:', ':curry:', ':fried_shrimp:', ':bento:',
':sushi:', ':fish_cake:', ':rice_ball:', ':rice_cracker:', ':rice:', ':ramen:',
':stew:', ':oden:', ':dango:', ':egg:', ':bread:', ':doughnut:', ':custard:',
':icecream:', ':ice_cream:', ':shaved_ice:', ':birthday:', ':cake:', ':cookie:',
':chocolate_bar:', ':candy:', ':lollipop:', ':honey_pot:', ':apple:',
':green_apple:', ':tangerine:', ':lemon:', ':cherries:', ':grapes:',
':watermelon:', ':strawberry:', ':peach:', ':melon:', ':banana:', ':pear:',
':pineapple:', ':sweet_potato:', ':eggplant:', ':tomato:', ':corn:', ':poop:', )


class User(object):
    def __init__(self):
        self.name = fake.name()
        self.user_id = generate_monzo_id('cus')


class Account(object):
    def __init__(self, user, currency):
        self.account_id = generate_monzo_id('acc')
        self.description = '%s\'s %s' % (user.name, fake.word().title())
        self.created = fake.date_time_between(start_date='-1y', end_date='now')
        self.balance = random.randint(1000, 100000)
        self.spend_today = random.randint(100, 10000)

        if currency == 'random':
            self.currency = fake.currency_code()
        else:
            self.currency = currency


class Address(object):
    def __init__(self):
        self.address = '%s %s' % (fake.building_number(), fake.street_name())
        self.city = fake.city()
        self.country = fake.country_code()
        self.postcode = fake.zipcode()
        self.latitude = fake.latitude()
        self.longitude = fake.longitude()
        self.region = fake.state()


class MerchantGroup(object):
    def __init__(self):
        self.merchant_group_id = generate_monzo_id('grp')


class Merchant(object):
    def __init__(self, groups):
        self.merchant_id = generate_monzo_id('merch')
        self.merchant_group_id = random.choice(groups).merchant_group_id
        self.created = fake.date_time_between(start_date='-1y', end_date='now')
        self.logo = fake.image_url(width=42, height=42)
        self.address = Address()
        self.emoji = emoji.emojize(random.choice(EMOJI))
        self.category = random.choice(CATEGORIES)
        self.name = fake.company()

        if random.random() < 0.1:
            self.name += ' ' + fake.company_suffix()


class Transaction(object):
    def __init__(self, account, merchants):
        self.transaction_id = generate_monzo_id('tx')
        self.description = fake.sentence()
        self.currency = account.currency
        self.merchant = random.choice(merchants)
        self.metadata = {}
        self.account_balance = random.randint(1000, 100000)
        self.amount = random.randint(-10000, -100)
        self.created = fake.date_time_between(start_date='-1y', end_date='now')

        if random.random() < 0.9:
            self.settled = fake.date_time_between(
                start_date=self.created,
                end_date='now',
            )
        else:
            self.settled = None

        if random.random() < 0.95:
            self.is_load = True
            self.amount *= -1
        else:
            self.is_load = False

        if random.random() < 0.1:
            self.notes = fake.paragraph()
        else:
            self.notes = ""


def generate_token():
    return fake.sha1(raw_output=False)


def generate_monzo_id(kind):
    return '%s_%d%s' % (
        kind,
        fake.pyint(),
        fake.pystr(min_chars=20, max_chars=20),
    )



from urllib import parse
from flask import Flask, json, request, redirect, abort

from .database import db
from .datatypes import generate_token


server = Flask('monzo-api-stub')


@server.after_request
def access_control_headers(response):
    response.headers['Access-Control-Allow-Origin'] = '*'
    response.headers['Access-Control-Allow-Headers'] = 'Authorization'
    return response


@server.route('/')
def root():
    state = request.args['state']
    redirect_uri = request.args['redirect_uri']
    code = generate_token()

    uri = parse.urlparse(redirect_uri)
    query = parse.parse_qsl(uri.query) + [('state', state), ('code', code)]
    uri = uri._replace(query=parse.urlencode(query))

    return redirect(parse.urlunparse(uri))


@server.route('/oauth2/token/', methods=['POST'])
def oauth2_token():
    return json.dumps({
        'access_token': generate_token(),
        'client_id': request.form['client_id'],
        'expires_in': 21600,
        'refresh_token': generate_token(),
        'token_type': 'Bearer',
        'user_id': db['user'].user_id,
    })


@server.route('/ping/whoami/')
def whoami():
    return json.dumps({
        'authenticated': True,
        'client_id': generate_token(),
        'user_id': db['user'].user_id,
    })


@server.route('/accounts/')
def accounts():
    return json.dumps({
        'accounts': [
            {
                'id': account.account_id,
                'description': account.description,
                'created': rfc3339(account.created),
            }
            for account in db['accounts'].values()
        ]
    })


@server.route('/balance/')
def balance():
    account_id = request.args['account_id']

    try:
        account = db['accounts'][account_id]
    except KeyError:
        abort(400)

    return json.dumps({
        'balance': account.balance,
        'spend_today': account.spend_today,
        'currency': account.currency,
    })


@server.route('/transactions/<transaction_id>/')
def transaction(transaction_id):
    # Search all the accounts for the matching transaction
    transaction = None
    for _, transactions in db['transactions'].items():
        transaction = transactions.get(transaction_id)
        if transaction:
            break

    if not transaction:
        abort(404)

    expand_merchant = request.args.get('expand[]') == 'merchant'

    return json.dumps({
        'transaction': transaction_dict(transaction, expand_merchant),
    })


@server.route('/transactions/')
def transactions():
    account_id = request.args['account_id']

    try:
        transactions = db['transactions'][account_id]
    except KeyError:
        abort(400)

    expand_merchant = request.args.get('expand[]') == 'merchant'

    return json.dumps({
        'transactions': [
            transaction_dict(t, expand_merchant) for t in transactions.values()
        ],
    })


def transaction_dict(t, expand_merchant):
    if expand_merchant:
        merchant = merchant_dict(t.merchant)
    else:
        merchant = t.merchant.merchant_id

    return {
        'account_balance': t.account_balance,
        'amount': t.amount,
        'created': rfc3339(t.created),
        'currency': t.currency,
        'description': t.description,
        'id': t.transaction_id,
        'merchant': merchant,
        'metadata': t.metadata,
        'notes': t.notes,
        'is_load': t.is_load,
        'settled': rfc3339(t.settled),
    }


def merchant_dict(m):
    return {
        'address': {
            'address': m.address.address,
            'city': m.address.city,
            'country': m.address.country,
            'latitude': float(m.address.latitude),
            'longitude': float(m.address.longitude),
            'postcode': m.address.postcode,
            'region': m.address.region,
        },
        'created': rfc3339(m.created),
        'group_id': m.merchant_group_id,
        'id': m.merchant_id,
        'logo': m.logo,
        'emoji': m.emoji,
        'name': m.name,
        'category': m.category,
    }


def rfc3339(d):
    if d is None:
        return ''
    return d.strftime('%Y-%m-%dT%H:%M:%SZ')



import click

from . import database
from .server import server

@click.command()
@click.option('--host', default='127.0.0.1', help='Address to bind to.')
@click.option('--port', default=3000, help='Port for the HTTP server.')
@click.option('--num-accounts', default=1,
    help='Number of accounts to give the user')
@click.option('--currency', default='GBP',
    help='Currency to use, pass "random" to use random currencies')
def cli(host, port, num_accounts, currency):
    """Stub Monzo API"""
    database.init(num_accounts, currency)
    server.run(host=host, port=port)

if __name__ == '__main__':
    cli()



from setuptools import setup

readme = open('README.rst').read()

setup(
    name='monzo-api-stub',
    version='0.1.2',
    description="Stub API for Monzo",
    long_description=readme,
    author="Dan Palmer",
    author_email='dan@danpalmer.me',
    license='MIT',
    url='https://github.com/danpalmer/monzo-api-stub',
    packages=[
        'monzoapistub',
    ],
    install_requires=[
        'click>=6.6',
        'flask>=0.11',
        'faker>=0.8.6',
        'emoji>=0.3.9',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
    ],
    entry_points={
        'console_scripts': [
            'monzo-api = monzoapistub.cli:cli',
        ],
    },
)
